(function (Drupal, once) {
  'use strict';

  Drupal.behaviors.eventPlatformScheduler = {
    attach: function (context) {

      const unassign = document.querySelector('.scheduler--unassign');

      // Activate empty slots as drag targets.
      once('eventPlatformScheduler', '#scheduler-table .empty', context).forEach(function (element) {
        makDragTarget(element);
      });

      // Activate unassign slots as a drag target.
      once('eventPlatformScheduler', '#scheduler--sessions .scheduler--unassign', context).forEach(function (element) {
        element.addEventListener('dragover', dragOver);
        element.addEventListener('dragenter', dragEnter);
        element.addEventListener('dragleave', dragLeave);
        element.addEventListener('drop', dragDropUnassign);
      });

      // Activate sessions as draggable.
      once('eventPlatformScheduler', '.session', context).forEach(function (element) {
        element.addEventListener('dragstart', dragStart);
        element.addEventListener('dragend', dragEnd);
        if (element.dataset.room && element.dataset.timeslot) {
          const cell = document.querySelector('td[data-room="' + element.dataset.room + '"][data-timeslot="' + element.dataset.timeslot + '"]');
          if (cell) {
            // const data = e.dataTransfer.getData("text/plain");
            // const node = document.getElementById(data)
            // e.target.appendChild(node);
            cell.appendChild(element);
            cell.classList.remove("empty");
            cell.classList.add("filled");
            removeDragTarget(cell);
          }
        }
      });

      // @todo move any sessions already scheduled.

      function makDragTarget(element) {
        element.addEventListener('dragover', dragOver);
        element.addEventListener('dragenter', dragEnter);
        element.addEventListener('dragleave', dragLeave);
        element.addEventListener('drop', dragDrop);
      }

      function removeDragTarget(element) {
        element.removeEventListener('dragover', dragOver);
        element.removeEventListener('dragenter', dragEnter);
        element.removeEventListener('dragleave', dragLeave);
        element.removeEventListener('drop', dragDrop);
      }

      // Drag Functions
      function dragStart(e) {
        this.classList.add("dragging");
        e.dataTransfer.setData("text/plain", e.target.id);
        const slot = this.closest(".filled");
        if (slot) {
          makDragTarget(slot);
        }
        if (this.dataset.room || this.dataset.timeslot) {
          unassign.classList.remove('hidden');
        }
      }

      function dragEnd() {
        this.classList.remove("dragging");
        unassign.classList.add('hidden');
      }

      function dragOver(e) {
        e.preventDefault();
        e.dataTransfer.dropEffect = "move";
      }

      function dragEnter(e) {
        e.preventDefault();
        this.classList.add("dragover");
      }

      function dragLeave() {
        this.classList.remove("dragover");
      }

      function dragDrop(e) {
        // Get the id of the target and add the moved element to the target's DOM
        const data = e.dataTransfer.getData("text/plain");
        const node = document.getElementById(data)
        e.target.appendChild(node);
        // Replace classes.
        this.classList.remove("dragover");
        this.classList.remove("empty");
        this.classList.add("filled");
        removeDragTarget(this);
        // Assemble the data necessary to update the node.
        const nid = node.dataset.nid;
        const room = this.dataset.room;
        const timeslot = this.dataset.timeslot;
        if (nid && room && timeslot) {
          // console.log("Target: " + nid + "-" + room + "-" + timeslot);
          const path =  '/admin/event-details/scheduler/assign/' + nid + '/' + room + '/' + timeslot;
          fetch(path)
            .then(response => response.text())
            .then(text => console.log(text));
          // @todo show feedback to user.
          // Add data attributes to the session.
          node.dataset.room = room;
          node.dataset.timeslot = timeslot;
        }
      }

      function dragDropUnassign(e) {
        // Get the id of the target and returbn the element to the list.
        const data = e.dataTransfer.getData("text/plain");
        const node = document.getElementById(data)
        e.target.parentElement.appendChild(node);
        const nid = node.dataset.nid;
        if (nid) {
          const path = '/admin/event-details/scheduler/unassign/' + nid;
          fetch(path)
            .then(response => response.text())
            .then(text => console.log(text));
          // @todo show feedback to user.
        }
        // Reset data attributes on the session.
        node.dataset.room = '';
        node.dataset.timeslot = '';
      }

    }
  };
}(Drupal, once));
